﻿/// <reference path="/resources/JQuery/jquery-1.7-vsdoc.js" />

var getHashValue = function (key) {
    key = key.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
    var regex = new RegExp("[\\#&]" + key + "=([^&#]*)");
    var qs = regex.exec(window.location.hash);
    if (qs == null) {
        return undefined;
    }
    else {
        return decodeURIComponent(qs[1]);
    }
};

var redirectWithHashValues = function (o) {
    // valid properties behind the hash sign 
    o.s = (o.s === undefined) ? getHashValue('s') : o.s;
    o.p = (parseInt(o.p) === NaN) ? getHashValue('p') : o.p;

    // iterate valid properties and mimic a query string with a # instead of a ?
    var url = window.location.protocol + '//' + window.location.host + window.location.pathname + '#';
    $.each(o, function (propName, propValue) {
        if (propValue !== undefined && propValue !== 0) {
            url += propName + '=' + encodeURIComponent(propValue) + '&';
        }
    });
    // remove trailing # if there are no key/value pairs, or & if there is at least one
    url = url.substring(0, url.length - 1);

    window.location = url;
};

proget_browse = function (o) {
    var pos = 0;
    var container = $('.packageList');
    var totalCount = null;
    var countUrl;
    var versionFilterQuery = o.includePrerelease ? 'IsAbsoluteLatestVersion' : 'IsLatestVersion';

    var ajaxReloadTotalCount = function (async) {
        if (getHashValue('s')) {
            countUrl = o.apiUrl + "/Search()/$count?$filter=" + versionFilterQuery + "&searchTerm='" + getHashValue('s') + "'&targetFramework=''&includePrerelease=" + o.includePrerelease;
        } else {
            countUrl = o.apiUrl + '/Packages/$count?$filter=' + versionFilterQuery;
        }
        $.ajax({
            url: countUrl,
            cache: false,
            async: async,
            success: function (data) {
                totalCount = data;
                var orig = $('.packageListTitle').text();
                if (parseInt(totalCount) > 0) {
                    $('.packageListTitle').text(orig + ' of ' + totalCount);
                }
            }
        });
    };
    ajaxReloadTotalCount(true);

    var updateTitle = function (reloadTotalCount) {
        if (reloadTotalCount) {
            ajaxReloadTotalCount(false);
        }

        var titleText = 'Displaying packages ' + (pos + 1) + '-' + Math.min(totalCount, pos + 20);
        if (parseInt(totalCount) > 0) {
            titleText += ' of ' + totalCount;
        } else {
            titleText = 'No packages found.';
            $('.packageNavLinks').hide();
        }
        $('.packageListTitle').text(titleText);
    };

    var getPackages = function () {
        pos = parseInt(getHashValue('p') || 0);
        var url;
        if (getHashValue('s')) {
            url = o.apiUrl + '/Search()/?$format=json&$filter=' + versionFilterQuery + '&$orderby=DownloadCount%20desc&$top=20&$skip=' + pos + '&searchTerm=\'' + encodeURIComponent(getHashValue('s')) + '\'&targetFramework=\'\'&includePrerelease=' + o.includePrerelease;
        } else {
            url = o.apiUrl + '/Packages/?$format=json&$filter=' + versionFilterQuery + '&$orderby=DownloadCount%20desc&$top=20&$skip=' + pos;
        }

        $('.loading').show();
        container.fadeOut('fast');
        container.children().remove();
        $('.packageNavLinks').hide();
        $('.packageNavLinks > a').addClass('inactive');
        $.ajax({
            url: url,
            cache: false,
            success: function (data, textStatus, jqXHR) {

                if (jqXHR.getResponseHeader('x-pgconnectorerror')) {
                    $(".info-box.warning").show();
                    $(".info-box.warning p").text('There was an error with one or more of your connectors: ' + jqXHR.getResponseHeader('x-pgconnectorerror'));
                }

                container.hide();
                container.children().remove();
                var packages = data.d.results;
                for (var i = 0; i < packages.length; i++) {
                    var packageHtml = '<div><hr/><div class="iconPart">';
                    if (!packages[i].IsProGetHosted) {
                        packageHtml += '<img src="/0x44/InedoLib/Inedo.Web.Controls.IconImage/ProcessImageRequest/InedoIcons/Virtuallnk/24x24/Connection.png"/>';
                    }

                    if (o.includePrerelease)
                        packageHtml += '</div><div class="descriptionPart"><h3><a href="' + o.baseUrl + '/' + packages[i].Id + '/' + packages[i].Version + '">' + packages[i].Id + ' ' + packages[i].Version + '</a></h3><p>' + packages[i].Description + '</p></div></div>';
                    else
                        packageHtml += '</div><div class="descriptionPart"><h3><a href="' + o.baseUrl + '/' + packages[i].Id + '">' + packages[i].Id + '</a></h3><p>' + packages[i].Description + '</p></div></div>';

                    var newPackage = $(packageHtml);

                    if (packages[i].IconUrl) {
                        newPackage.find('.iconPart').css('background-image', 'url(\'' + packages[i].IconUrl + '\')');
                    }

                    container.append(newPackage);
                }

                if (pos > 0) {
                    $('.packageNavLinks a:first-child').removeClass('inactive');
                }

                if (packages.length >= 20) {
                    $('.packageNavLinks a:last-child').removeClass('inactive');
                }

                container.fadeIn('fast');

                $('.loading').hide();

                $('.packageNavLinks').show();

                updateTitle(true);
            },
            error: function (jqXHR, textStatus, errorThrown) {
                $(".info-box.error").show();
                $(".info-box.error p").text('An unexpected error has occurred: ' + jqXHR.responseText);
                $('.loading').hide();
                console.log(jqXHR);
                console.log(textStatus);
                console.log(errorThrown);
            }
        });
    };

    $('.packageNavLinks a:first-child').click(function () {
        if (!$(this).hasClass('inactive')) {
            pos -= 20;
            if (pos < 0) {
                pos = 0;
            }
            redirectWithHashValues({ p: pos });
        }
        return false;
    });

    $('.packageNavLinks a:last-child').click(function () {
        if (!$(this).hasClass('inactive')) {
            pos += 20;
            redirectWithHashValues({ p: pos });
        }
        return false;
    });

    $('form').submit(function () {

        var searchTerm = $('.package-search input').val();
        redirectWithHashValues({ s: searchTerm });

        return false;
    });

    $('.package-search img').click(function () {
        $('form').submit();
    });

    $(window).bind('hashchange', function () {
        getPackages();
    });

    getPackages();
};